==> cg_flake8_reporter-0.1.0-py3-none-any.whl: cg_flake8_reporter/plugin.py <==
"""Implementation of CodeGrade's formatter for flake8 reporting directly to
AutoTest v2.
"""
import os
import json
import typing as t
import argparse
from pathlib import Path
from fractions import Fraction

import flake8  # type: ignore[import]

# The message size limit for SQS messages is 256KB. We set it a little bit less
# here than 256KB to leave enough space for the wrapping message. Large
# structured output messages are stored on S3 so we don't have to stay with the
# WebSocket server message size limit of 128KB for these messages.
MESSAGE_SIZE_LIMIT = 250_000


def json_dumps(obj: object) -> str:
    """Dump an object to JSON without any extra formatting.
    """
    # Make sure only ASCII characters are used so that the string length as
    # python's `len` function reports it is equal to the string's byte length.
    # Do not insert spaces after each separator.
    return json.dumps(obj, ensure_ascii=True, separators=(',', ':'))


class CustomFormatter(flake8.formatting.base.BaseFormatter):
    """
    Custom Formatter for flake8. 
    Formats the errors/warnings as JSON objects.
    Also calculates a score based on severity of errors/warnings.
    """
    def __init__(self, options: argparse.Namespace) -> None:
        """
        Initialize buffer for comments, score, and points deducted for each severity level.
        """
        super().__init__(options)
        self._buffer: t.Mapping[str,
                                t.Any] = {"tag": "comments", "comments": {}}
        self._score = Fraction(1, 1)
        self._buffer_size = 0
        self._points_deducted = {
            'info': Fraction(0, 1), 'warning': Fraction(0, 1), 'error':
                Fraction(0, 1), 'fatal': Fraction(0, 1)
        }
        # pylint: disable=consider-using-with
        try:
            self._file = open(options.cg_flake8_fd, 'wb', buffering=0)
        except OSError as e:
            raise IOError(
                f'Could not open file descriptor {options.cg_flake8_fd}'
            ) from e

        self._base_path = Path(options.cg_base_path)

        if options.cg_points_deducted:
            self._points_deducted = {
                severity: Fraction(int(points), 100)
                for severity, points in (
                    item.split(':')
                    for item in options.cg_points_deducted.split(',')
                )
            }

    @classmethod
    def add_options(cls, parser: flake8.options.manager.OptionManager) -> None:
        """Adds the `cg-points-deducted` option to flake8 to set the deduction
        percentages for each of the severity levels.
        Adds the `cg-flake8-fd` option to flake8 to set the file descriptor to
        which to output the reports.
        """
        parser.add_option(
            '--cg-points-deducted',
            dest='cg_points_deducted',
            action='store',
            type=str,
            parse_from_config=True,
            help="""
Points deducted for each severity level. Should be a string of
the form 'info:1,warning:5,error:10,fatal:10'
            """,
        )
        parser.add_option(
            '--cg-flake8-fd',
            dest='cg_flake8_fd',
            action='store',
            type=int,
            metavar='file descriptor',
            default=1,
            help="File descriptor to write the results to.",
        )
        parser.add_option(
            '--cg-base-path',
            dest='cg_base_path',
            action='store',
            type=str,
            default='/',
            help="""
The base path of the reported files. Files that are not children of this path
will not be reported.
            """
        )

    def format(self, error: flake8.violation.Violation):
        """
        Format the error/warning and add to buffer.
        Also update the score based on error severity.
        """
        try:
            abs_path = os.path.abspath(error.filename)
            relative_path = Path(abs_path).relative_to(self._base_path)
        except ValueError:
            return

        code = error.code
        if code.startswith('E') or code.startswith('F'):
            severity = 'error'
        elif code.startswith('W'):
            severity = 'warning'
        elif code.startswith('C') or code.startswith('N8'):  # pragma: no cover
            severity = 'info'

        if severity in self._points_deducted:
            self._score = max(
                self._score - self._points_deducted[severity], Fraction(0)
            )

        err_item = {
            "code": error.code,
            "severity": severity,
            "loc": {
                "start": {
                    "line": error.line_number,
                    "column": error.column_number,
                },
                "end": {
                    "line": error.line_number,
                    "column": error.column_number,
                },
            },
            "msg": error.text,
        }

        file_comments = self._buffer["comments"].get(str(relative_path))
        if file_comments is None:
            file_comments = {
                "filename": str(relative_path),
                "origin": "Flake8",
                "items": [],
            }
            self._buffer["comments"][str(relative_path)] = file_comments

        file_comments["items"].append(err_item)
        self._buffer_size += len(json_dumps(err_item))

        if self._buffer_size >= MESSAGE_SIZE_LIMIT:
            self.flush()

    def flush(self):
        """
        Write the comments to file descriptor and clear the buffer.
        """
        self._buffer["comments"] = list(self._buffer["comments"].values())
        data = json_dumps(self._buffer) + '\n'

        try:
            self._file.write(data.encode('utf8'))
        except OSError as e:
            raise IOError(
                f'Could not write comments to file descriptor {self._file.fileno()}'
            ) from e
        finally:
            self._buffer = {"tag": "comments", "comments": {}}
            self._buffer_size = 0

    def stop(self):
        """
        At the end, flush the remaining comments and write the score.
        """
        self.flush()
        data = json_dumps({"tag": "points", "points": str(self._score)})

        try:
            self._file.write(data.encode('utf8'))
        except OSError as e:
            raise IOError(
                f'Could not write score to file descriptor {self._file.fileno()}'
            ) from e

==> cg_flake8_reporter-2023.6.12.1304-py3-none-any.whl: cg_flake8_reporter/plugin.py <==
"""Implementation of CodeGrade's formatter for flake8 reporting directly to
AutoTest v2.
"""
import os
import json
import typing as t
import argparse
from pathlib import Path
from fractions import Fraction

import flake8  # type: ignore[import]

# The message size limit for SQS messages is 256KB. We set it a little bit less
# here than 256KB to leave enough space for the wrapping message. Large
# structured output messages are stored on S3 so we don't have to stay with the
# WebSocket server message size limit of 128KB for these messages.
MESSAGE_SIZE_LIMIT = 250_000


def json_dumps(obj: object) -> str:
    """Dump an object to JSON without any extra formatting.
    """
    # Make sure only ASCII characters are used so that the string length as
    # python's `len` function reports it is equal to the string's byte length.
    # Do not insert spaces after each separator.
    return json.dumps(obj, ensure_ascii=True, separators=(',', ':'))


class CustomFormatter(flake8.formatting.base.BaseFormatter):
    """
    Custom Formatter for flake8.
    Formats the errors/warnings as JSON objects.
    Also calculates a score based on severity of errors/warnings.
    """
    def __init__(self, options: argparse.Namespace) -> None:
        """
        Initialize buffer for comments, score, and points deducted for each severity level.
        """
        super().__init__(options)
        self._buffer: t.MutableMapping[str, t.Any] = {
            "tag": "comments", "comments": {}
        }
        self._score = Fraction(1, 1)
        self._buffer_size = 0
        self._points_deducted = {
            'info': Fraction(0, 1),
            'warning': Fraction(0, 1),
            'error': Fraction(0, 1),
        }
        # pylint: disable=consider-using-with
        self._file = open(options.cg_flake8_fd, 'wb', buffering=0)

        self._base_path = Path(options.cg_base_path)

        if options.cg_points_deducted:
            self._points_deducted = {
                severity: Fraction(int(points), 100)
                for severity, points in (
                    item.split(':')
                    for item in options.cg_points_deducted.split(',')
                )
            }

    @classmethod
    def add_options(cls, parser: flake8.options.manager.OptionManager) -> None:
        """Adds the `cg-points-deducted` option to flake8 to set the deduction
        percentages for each of the severity levels.
        Adds the `cg-flake8-fd` option to flake8 to set the file descriptor to
        which to output the reports.
        """
        parser.add_option(
            '--cg-points-deducted',
            dest='cg_points_deducted',
            action='store',
            type=str,
            parse_from_config=True,
            help="""
Points deducted for each severity level. Should be a string of
the form 'info:1,warning:5,error:10'
            """,
        )
        parser.add_option(
            '--cg-flake8-fd',
            dest='cg_flake8_fd',
            action='store',
            type=int,
            metavar='file descriptor',
            default=1,
            help="File descriptor to write the results to.",
        )
        parser.add_option(
            '--cg-base-path',
            dest='cg_base_path',
            action='store',
            type=str,
            default='/',
            help="""
The base path of the reported files. Files that are not children of this path
will not be reported.
            """
        )

    def format(self, error: flake8.violation.Violation) -> None:
        """
        Format the error/warning and add to buffer.
        Also update the score based on error severity.
        """
        try:
            abs_path = os.path.abspath(error.filename)
            relative_path = Path(abs_path).relative_to(self._base_path)
        except ValueError:
            return

        code = error.code
        if code.startswith('E') or code.startswith('F'):
            severity = 'error'
        elif code.startswith('W'):
            severity = 'warning'
        elif code.startswith('C') or code.startswith('N8'):
            severity = 'info'

        if severity in self._points_deducted:
            self._score = max(
                self._score - self._points_deducted[severity], Fraction(0)
            )

        err_item = {
            "code": error.code,
            "severity": severity,
            "loc": {
                "start": {
                    "line": error.line_number,
                    "column": error.column_number,
                },
                "end": {"line": error.line_number, },
            },
            "msg": error.text,
        }

        added_size = len(json_dumps(err_item))

        if self._buffer_size + added_size >= MESSAGE_SIZE_LIMIT:
            self.flush()

        file_comments = self._buffer["comments"].get(str(relative_path))
        if file_comments is None:
            file_comments = {
                "filename": str(relative_path),
                "origin": "Flake8",
                "items": [],
            }
            added_size += len(json_dumps(file_comments))

        file_comments["items"].append(err_item)
        self._buffer["comments"][str(relative_path)] = file_comments
        self._buffer_size += added_size

    def flush(self) -> None:
        """
        Write the comments to file descriptor and clear the buffer.
        """
        self._buffer["comments"] = list(self._buffer["comments"].values())
        data = json_dumps(self._buffer) + '\n'

        self._file.write(data.encode('utf8'))
        self._buffer = {"tag": "comments", "comments": {}}
        self._buffer_size = len(json_dumps(self._buffer))

    def stop(self) -> None:
        """
        At the end, flush the remaining comments and write the score.
        """
        self.flush()
        data = json_dumps({"tag": "points", "points": str(self._score)})
        self._file.write(data.encode('utf8'))
